#!/usr/bin/python3
"""
Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 GNU Public License.
"""

# from importlib.metadata import version as __version__

DEPENDS_SUBSTVARS = {
    "fortran": "${fortran:Depends}",
}
PKG_NAME_TPLS = {
    "fortran": ("libfortran-", "fortran."),
}
# options.command = {"fortran": "dh-fortran"}
