#!/usr/bin/python3
"""
Test Driver for dh-fortran/cli.py

Copyright (C) 2025 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 Gnu Public License.
"""

import dhfortran.cli as cli


def test_validate_flavor():
    cli.validate_flavor(None)    

def test_validate_fc_none():
    cli.validate_compiler(None)
    
def test_get_env():
    pass



if __name__ == "__main__":
    import pytest

    pytest.main()
