#!/usr/bin/python3
"""
Test Driver for dh-fortran/main.py

Copyright (C) 2025-2026 Alastair McKinstry <mckinstry@debian.org>
Released under the GPL-3 Gnu Public License.
"""

from click.testing import CliRunner
import dhfortran.compilers as cmplrs
import dhfortran.debhelper as dh


def test_rework_flang21():
    out = cmplrs.rework_flags('flang-21',None)
    breakpoint()
    print(out)

def test_fcflags_flang21():
    out = cmplrs.get_fc_flags('flang-21',None)
    print(out)
